@@ -68,7 +68,7 @@ def _rc_path():
     return expanded_rc_path
 
 default_port = '22'  # hurr durr
-default_ssh_config_path = '~/.ssh/config'
+default_ssh_config_path = os.path.join(os.path.expanduser('~'), '.ssh', 'config')
 
 # Options/settings which exist both as environment keys and which can be set on
 # the command line, are defined here. When used via `fab` they will be added to
@@ -141,6 +141,24 @@ env_options = [
         help="gateway host to connect through"
     ),
 
+    make_option('--gss-auth',
+        action='store_true',
+        default=None,
+        help="Use GSS-API authentication"
+    ),
+
+    make_option('--gss-deleg',
+        action='store_true',
+        default=None,
+        help="Delegate GSS-API client credentials or not"
+    ),
+
+    make_option('--gss-kex',
+        action='store_true',
+        default=None,
+        help="Perform GSS-API Key Exchange and user authentication"
+    ),
+
     make_option('--hide',
         metavar='LEVELS',
         help="comma-separated list of output levels to hide"
@@ -217,6 +235,11 @@ env_options = [
         help="reject unknown hosts"
     ),
 
+    make_option('--sudo-password',
+        default=None,
+        help="password for use with sudo only",
+    ),
+
     make_option('--system-known-hosts',
         default=None,
         help="load system known_hosts file before reading user known_hosts"
@@ -243,6 +266,12 @@ env_options = [
         help="skip over hosts that can't be reached"
     ),
 
+    make_option('--skip-unknown-tasks',
+        action="store_true",
+        default=False,
+        help="skip over unknown tasks"
+    ),
+
     make_option('--ssh-config-path',
         default=default_ssh_config_path,
         metavar='PATH',
@@ -318,6 +347,9 @@ env = _AttributeDict({
     'effective_roles': [],
     'exclude_hosts': [],
     'gateway': None,
+    'gss_auth': None,
+    'gss_deleg': None,
+    'gss_kex': None,
     'host': None,
     'host_string': None,
     'lcwd': '',  # Must be empty string, not None, for concatenation purposes
@@ -333,7 +365,9 @@ env = _AttributeDict({
     'roledefs': {},
     'shell_env': {},
     'skip_bad_hosts': False,
+    'skip_unknown_tasks': False,
     'ssh_config_path': default_ssh_config_path,
+    'sudo_passwords': {},
     'ok_ret_codes': [0],     # a list of return codes that indicate success
     # -S so sudo accepts passwd via stdin, -p with our known-value prompt for
     # later detection (thus %s -- gets filled with env.sudo_prompt at runtime)
@@ -419,10 +453,11 @@ output = _AliasDict({
     'running': True,
     'stdout': True,
     'stderr': True,
+    'exceptions': False,
     'debug': False,
     'user': True
 }, aliases={
-    'everything': ['warnings', 'running', 'user', 'output'],
+    'everything': ['warnings', 'running', 'user', 'output', 'exceptions'],
     'output': ['stdout', 'stderr'],
     'commands': ['stdout', 'running']
 })
