@@ -16,8 +16,8 @@ from fabric.io import output_loop
 import fabric.network  # So I can call patch_object correctly. Sigh.
 from fabric.state import env, output, _get_system_username
 from fabric.operations import run, sudo, prompt
-from fabric.exceptions import NetworkError
 from fabric.tasks import execute
+from fabric.api import parallel
 from fabric import utils # for patching
 
 from utils import *
@@ -547,6 +547,27 @@ class TestConnections(FabricTest):
             execute(subtask, hosts=['nope.nonexistent.com'])
 
 
+@parallel
+def parallel_subtask():
+    run("This should never execute")
+
+class TestParallelConnections(FabricTest):
+    @aborts
+    def test_should_abort_when_cannot_connect(self):
+        """
+        By default, connecting to a nonexistent server should abort.
+        """
+        with hide('everything'):
+            execute(parallel_subtask, hosts=['nope.nonexistent.com'])
+
+    def test_should_warn_when_skip_bad_hosts_is_True(self):
+        """
+        env.skip_bad_hosts = True => execute() skips current host
+        """
+        with settings(hide('everything'), skip_bad_hosts=True):
+            execute(parallel_subtask, hosts=['nope.nonexistent.com'])
+
+
 class TestSSHConfig(FabricTest):
     def env_setup(self):
         super(TestSSHConfig, self).env_setup()
@@ -592,7 +613,7 @@ class TestSSHConfig(FabricTest):
         """
         Global Port should NOT override nondefault env.port
         """
-        with settings(port="777"):
+        with settings(port="777", use_ssh_config=False):
             eq_(normalize("localhost")[2], "777")
 
     def test_specific_port_with_default_env(self):
@@ -619,7 +640,7 @@ class TestSSHConfig(FabricTest):
         expect_call=True))
     def test_warns_with_bad_config_file_path(self):
         # use_ssh_config is already set in our env_setup()
-        with settings(ssh_config_path="nope_bad_lol"):
+        with settings(hide('everything'), ssh_config_path="nope_bad_lol"):
             normalize('foo')
 
     @server()
