@@ -8,23 +8,23 @@ import random
 import sys
 
 import fabric
-from fabric import tasks
-from fabric.tasks import WrappedCallableTask, execute, Task
-from fabric.api import run, env, settings, hosts, roles, hide, parallel
+from fabric.tasks import WrappedCallableTask, execute, Task, get_task_details
+from fabric.main import display_command
+from fabric.api import run, env, settings, hosts, roles, hide, parallel, task, runs_once, serial
 from fabric.network import from_dict
 from fabric.exceptions import NetworkError
 
-from utils import eq_, FabricTest, aborts, mock_streams
+from utils import eq_, FabricTest, aborts, mock_streams, support
 from server import server
 
 
 def test_base_task_provides_undefined_name():
-    task = tasks.Task()
+    task = Task()
     eq_("undefined", task.name)
 
 @raises(NotImplementedError)
 def test_base_task_raises_exception_on_call_to_run():
-    task = tasks.Task()
+    task = Task()
     task.run()
 
 class TestWrappedCallableTask(unittest.TestCase):
@@ -52,43 +52,48 @@ class TestWrappedCallableTask(unittest.TestCase):
     def test_allows_any_number_of_args(self):
         args = [i for i in range(random.randint(0, 10))]
         def foo(): pass
-        task = tasks.WrappedCallableTask(foo, *args)
+        task = WrappedCallableTask(foo, *args)
 
     def test_allows_any_number_of_kwargs(self):
         kwargs = dict([("key%d" % i, i) for i in range(random.randint(0, 10))])
         def foo(): pass
-        task = tasks.WrappedCallableTask(foo, **kwargs)
+        task = WrappedCallableTask(foo, **kwargs)
 
     def test_run_is_wrapped_callable(self):
         def foo(): pass
-        task = tasks.WrappedCallableTask(foo)
-        self.assertEqual(task.wrapped, foo)
+        task = WrappedCallableTask(foo)
+        eq_(task.wrapped, foo)
 
     def test_name_is_the_name_of_the_wrapped_callable(self):
         def foo(): pass
         foo.__name__ = "random_name_%d" % random.randint(1000, 2000)
-        task = tasks.WrappedCallableTask(foo)
-        self.assertEqual(task.name, foo.__name__)
+        task = WrappedCallableTask(foo)
+        eq_(task.name, foo.__name__)
+
+    def test_name_can_be_overridden(self):
+        def foo(): pass
+        eq_(WrappedCallableTask(foo).name, 'foo')
+        eq_(WrappedCallableTask(foo, name='notfoo').name, 'notfoo')
 
     def test_reads_double_under_doc_from_callable(self):
         def foo(): pass
         foo.__doc__ = "Some random __doc__: %d" % random.randint(1000, 2000)
-        task = tasks.WrappedCallableTask(foo)
-        self.assertEqual(task.__doc__, foo.__doc__)
+        task = WrappedCallableTask(foo)
+        eq_(task.__doc__, foo.__doc__)
 
     def test_dispatches_to_wrapped_callable_on_run(self):
         random_value = "some random value %d" % random.randint(1000, 2000)
         def foo(): return random_value
-        task = tasks.WrappedCallableTask(foo)
-        self.assertEqual(random_value, task())
+        task = WrappedCallableTask(foo)
+        eq_(random_value, task())
 
     def test_passes_all_regular_args_to_run(self):
         def foo(*args): return args
         random_args = tuple(
             [random.randint(1000, 2000) for i in range(random.randint(1, 5))]
         )
-        task = tasks.WrappedCallableTask(foo)
-        self.assertEqual(random_args, task(*random_args))
+        task = WrappedCallableTask(foo)
+        eq_(random_args, task(*random_args))
 
     def test_passes_all_keyword_args_to_run(self):
         def foo(**kwargs): return kwargs
@@ -96,31 +101,32 @@ class TestWrappedCallableTask(unittest.TestCase):
         for i in range(random.randint(1, 5)):
             random_key = ("foo", "bar", "baz", "foobar", "barfoo")[i]
             random_kwargs[random_key] = random.randint(1000, 2000)
-        task = tasks.WrappedCallableTask(foo)
-        self.assertEqual(random_kwargs, task(**random_kwargs))
+        task = WrappedCallableTask(foo)
+        eq_(random_kwargs, task(**random_kwargs))
 
     def test_calling_the_object_is_the_same_as_run(self):
         random_return = random.randint(1000, 2000)
         def foo(): return random_return
-        task = tasks.WrappedCallableTask(foo)
-        self.assertEqual(task(), task.run())
+        task = WrappedCallableTask(foo)
+        eq_(task(), task.run())
 
 
 class TestTask(unittest.TestCase):
     def test_takes_an_alias_kwarg_and_wraps_it_in_aliases_list(self):
         random_alias = "alias_%d" % random.randint(100, 200)
-        task = tasks.Task(alias=random_alias)
+        task = Task(alias=random_alias)
         self.assertTrue(random_alias in task.aliases)
 
     def test_aliases_are_set_based_on_provided_aliases(self):
         aliases = ["a_%d" % i for i in range(random.randint(1, 10))]
-        task = tasks.Task(aliases=aliases)
+        task = Task(aliases=aliases)
         self.assertTrue(all([a in task.aliases for a in aliases]))
 
     def test_aliases_are_None_by_default(self):
-        task = tasks.Task()
+        task = Task()
         self.assertTrue(task.aliases is None)
 
+
 # Reminder: decorator syntax, e.g.:
 #     @foo
 #     def bar():...
@@ -147,7 +153,7 @@ def test_decorator_closure_hiding():
     """
     from fabric.decorators import task, hosts
     def foo():
-        print env.host_string
+        print(env.host_string)
     foo = task(hosts("me@localhost")(foo))
     eq_(["me@localhost"], foo.hosts)
 
@@ -205,6 +211,15 @@ class TestExecute(FabricTest):
         """
         execute('thisisnotavalidtaskname')
 
+    def test_should_not_abort_if_task_name_not_found_with_skip(self):
+        """
+        should not abort if given an invalid task name
+        and skip_unknown_tasks in env
+        """
+        env.skip_unknown_tasks = True
+        execute('thisisnotavalidtaskname')
+        del env['skip_unknown_tasks']
+
     @with_fakes
     def test_should_pass_through_args_kwargs(self):
         """
@@ -401,3 +416,205 @@ class TestExecute(FabricTest):
             run = Fake(callable=True, expect_call=True)
         mytask = MyTask()
         execute(mytask)
+
+    @server(port=2200)
+    @server(port=2201)
+    def test_nested_execution_with_explicit_ports(self):
+        """
+        nested executions should work with defined ports
+        """
+
+        def expect_host_string_port():
+            eq_(env.port, '2201')
+            return "bar"
+
+        def expect_env_port():
+            eq_(env.port, '2202')
+
+        def expect_per_host_config_port():
+            eq_(env.port, '664')
+            run = execute(expect_default_config_port, hosts=['some_host'])
+            return run['some_host']
+
+        def expect_default_config_port():
+            # uses `Host *` in ssh_config
+            eq_(env.port, '666')
+            return "bar"
+
+        def main_task():
+            eq_(env.port, '2200')
+
+            execute(expect_host_string_port, hosts=['localhost:2201'])
+
+            with settings(port='2202'):
+                execute(expect_env_port, hosts=['localhost'])
+
+            with settings(
+                use_ssh_config=True,
+                ssh_config_path=support("ssh_config")
+            ):
+                run = execute(expect_per_host_config_port, hosts='myhost')
+
+            return run['myhost']
+
+        run = execute(main_task, hosts=['localhost:2200'])
+
+        eq_(run['localhost:2200'], 'bar')
+
+
+class TestExecuteEnvInteractions(FabricTest):
+    def set_network(self):
+        # Don't update env.host/host_string/etc
+        pass
+
+    @server(port=2200)
+    @server(port=2201)
+    def test_should_not_mutate_its_own_env_vars(self):
+        """
+        internal env changes should not bleed out, but task env changes should
+        """
+        # Task that uses a handful of features which involve env vars
+        @parallel
+        @hosts('username@127.0.0.1:2200', 'username@127.0.0.1:2201')
+        def mytask():
+            run("ls /simple")
+        # Pre-assertions
+        assertions = {
+            'parallel': False,
+            'all_hosts': [],
+            'host': None,
+            'hosts': [],
+            'host_string': None
+        }
+        for key, value in assertions.items():
+            eq_(env[key], value)
+        # Run
+        with hide('everything'):
+            result = execute(mytask)
+        eq_(len(result), 2)
+        # Post-assertions
+        for key, value in assertions.items():
+            eq_(env[key], value)
+
+    @server()
+    def test_should_allow_task_to_modify_env_vars(self):
+        @hosts('username@127.0.0.1:2200')
+        def mytask():
+            run("ls /simple")
+            env.foo = "bar"
+        with hide('everything'):
+            execute(mytask)
+        eq_(env.foo, "bar")
+        eq_(env.host_string, None)
+
+
+class TestTaskDetails(unittest.TestCase):
+    def test_old_style_task_with_default_args(self):
+        """
+        __details__() should print docstr for old style task methods with default args
+        """
+        def task_old_style(arg1, arg2, arg3=None, arg4='yes'):
+            '''Docstring'''
+        details = get_task_details(task_old_style)
+        eq_("Docstring\n"
+            "Arguments: arg1, arg2, arg3=None, arg4='yes'",
+            details)
+
+    def test_old_style_task_without_default_args(self):
+        """
+        __details__() should print docstr for old style task methods without default args
+        """
+
+        def task_old_style(arg1, arg2):
+            '''Docstring'''
+        details = get_task_details(task_old_style)
+        eq_("Docstring\n"
+            "Arguments: arg1, arg2",
+            details)
+
+    def test_old_style_task_without_args(self):
+        """
+        __details__() should print docstr for old style task methods without args
+        """
+
+        def task_old_style():
+            '''Docstring'''
+        details = get_task_details(task_old_style)
+        eq_("Docstring\n"
+            "Arguments: ",
+            details)
+
+    def test_decorated_task(self):
+        """
+        __details__() should print docstr for method with any number and order of decorations
+        """
+        expected = "\n".join([
+            "Docstring",
+            "Arguments: arg1",
+            ])
+
+        @task
+        def decorated_task(arg1):
+            '''Docstring'''
+
+        actual = decorated_task.__details__()
+        eq_(expected, actual)
+
+        @runs_once
+        @task
+        def decorated_task1(arg1):
+            '''Docstring'''
+
+        actual = decorated_task1.__details__()
+        eq_(expected, actual)
+
+        @runs_once
+        @serial
+        @task
+        def decorated_task2(arg1):
+            '''Docstring'''
+
+        actual = decorated_task2.__details__()
+        eq_(expected, actual)
+
+    def test_subclassed_task(self):
+        """
+        __details__() should print docstr for subclassed task methods with args
+        """
+
+        class SpecificTask(Task):
+            def run(self, arg1, arg2, arg3):
+                '''Docstring'''
+        eq_("Docstring\n"
+            "Arguments: self, arg1, arg2, arg3",
+            SpecificTask().__details__())
+
+    @mock_streams('stdout')
+    def test_multiline_docstring_indented_correctly(self):
+        """
+        display_command() should properly indent docstr for old style task methods
+        """
+
+        def mytask(arg1):
+            """
+            This is a multi line docstring.
+
+            For reals.
+            """
+        try:
+            with patched_context(fabric.state, 'commands', {'mytask': mytask}):
+                display_command('mytask')
+        except SystemExit: # ugh
+            pass
+        eq_(
+            sys.stdout.getvalue(),
+"""Displaying detailed information for task 'mytask':
+
+    This is a multi line docstring.
+    
+    For reals.
+    
+    Arguments: arg1
+
+"""
+        )
